program icz13_MemMappedFile;

{$R 'Resource.res' 'Resource.rc'}

uses
  Windows,
  Messages,
  CommDlg;

const
  IDM_OPEN = 1;
  IDM_SAVE = 2;
  IDM_EXIT = 3;

const
  ClassName    = 'Win32ASMFileMappingClass';
  AppName      = 'Win32 ASM File Mapping Example';
  MenuName     = 'FirstMenu';
  FilterString = 'All Files'#0'*.*'#0'Text Files'#0'*.txt'#0#0;

const
  MAXSIZE = 260;

var
  ofn   : TOpenFilename;
  buffer: array [0..MAXSIZE - 1] of Char;

var
  hMapFile   : THandle = 0;
  hFileRead  : THandle;
  hFileWrite : THandle;
  hMenu      : THandle;
  pMemory    : Pointer;
  SizeWritten: DWORD;

procedure CloseMapFile;
begin
  CloseHandle(hMapFile);
  hMapFile := 0;
  CloseHandle(hFileRead);
end;

function WndProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  Result := 0;
  case uMsg of
    WM_CREATE:
      begin
        hMenu := GetMenu(hWnd);
        ofn.lStructSize := SizeOf(TOpenFilename);
        ofn.hWndOwner   := hWnd;
        ofn.hInstance   := HInstance;
        ofn.lpstrFilter := FilterString;
        ofn.lpstrFile   := buffer;
        ofn.nMaxFile    := MAXSIZE;
      end;
    WM_DESTROY:
      begin
        if hMapFile <> 0 then
          CloseMapFile;
        PostQuitMessage(0);
      end;
    WM_COMMAND:
      if lParam = 0 then
      begin
        case LoWord(wParam) of
          IDM_OPEN:
            begin
              ofn.Flags := OFN_FILEMUSTEXIST or OFN_PATHMUSTEXIST or
                OFN_LONGNAMES or OFN_EXPLORER or OFN_HIDEREADONLY;
              if GetOpenFileName(ofn) then
              begin
                hFileRead := CreateFile(buffer, GENERIC_READ, 0, nil,
                  OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, 0);
                hMapFile := CreateFileMapping(hFileRead, nil, PAGE_READONLY,
                  0, 0, nil);
                SetWindowText(hWnd, PChar(@buffer[ofn.nFileOffset]));
                EnableMenuItem(hMenu, IDM_OPEN, MF_GRAYED);
                EnableMenuItem(hMenu, IDM_SAVE, MF_ENABLED);
              end;
            end;
          IDM_SAVE:
            begin
              ofn.Flags := OFN_LONGNAMES or OFN_EXPLORER or OFN_HIDEREADONLY;
              if GetSaveFileName(ofn) then
              begin
                hFileWrite := CreateFile(buffer, GENERIC_READ or
                  GENERIC_WRITE, FILE_SHARE_READ or FILE_SHARE_WRITE, nil,
                  CREATE_NEW, FILE_ATTRIBUTE_ARCHIVE, 0);
                pMemory := MapViewOfFile(hMapFile, FILE_MAP_READ, 0, 0, 0);
                WriteFile(hFileWrite, pMemory^, GetFileSize(hFileRead, nil),
                  SizeWritten, nil);
                UnmapViewOfFile(pMemory);
                CloseMapFile;
                CloseHandle(hFileWrite);
                SetWindowText(hWnd, AppName);
                EnableMenuItem(hMenu, IDM_OPEN, MF_ENABLED);
                EnableMenuItem(hMenu, IDM_SAVE, MF_GRAYED);
              end;
            end;
        else
          DestroyWindow(hWnd);
        end;
      end;
  else
    Result := DefWindowProc(hWnd, uMsg, wParam, lParam);
  end;
end;

var
  wc: TWndClassEx = (
    cbSize       : SizeOf(TWndClassEx);
    style        : CS_HREDRAW or CS_VREDRAW;
    lpfnWndProc  : @WndProc;
    cbClsExtra   : 0;
    cbWndExtra   : 0;
    hbrBackground: COLOR_WINDOW + 1;
    lpszMenuName : MenuName;
    lpszClassName: ClassName;
    hIconSm      : 0;
  );
  msg: TMsg;
  wnd: HWND;

begin
  wc.hInstance := HInstance;
  wc.hIcon     := LoadIcon(0, IDI_APPLICATION);
  wc.hCursor   := LoadCursor(0, IDC_ARROW);
  RegisterClassEx(wc);
  wnd := CreateWindowEx(WS_EX_CLIENTEDGE, ClassName, AppName,
    WS_OVERLAPPEDWINDOW, Integer(CW_USEDEFAULT), Integer(CW_USEDEFAULT), 300,
    200, 0, 0, HInstance, nil);
  ShowWindow(wnd, SW_SHOWNORMAL);
  UpdateWindow(wnd);
  while True do
  begin
    if not GetMessage(msg, 0, 0, 0) then
      Break;
    TranslateMessage(msg);
    DispatchMessage(msg);
  end;
  ExitCode := msg.wParam;
end.