program icz32_MDI;

{$R 'Resource.res' 'Resource.rc'}

uses
  Windows,
  Messages;

const
  IDR_MAINMENU  = 101;
  IDR_CHILDMENU = 102;
  IDM_EXIT      = 40001;
  IDM_TILEHORZ  = 40002;
  IDM_TILEVERT  = 40003;
  IDM_CASCADE   = 40004;
  IDM_NEW       = 40005;
  IDM_CLOSE     = 40006;

const
  ClassName          = 'MDIASMClass';
  MDIClientName      = 'MDICLIENT';
  MDIChildClassName  = 'Win32asmMDIChild';
  MDIChildTitle      = 'MDI Child';
  AppName            = 'Win32asm MDI Demo';
  ClosePromptMessage = 'Are you sure you want to close this window?';

var
  hMainMenu : HMENU;
  hwndClient: THandle;
  hChildMenu: HMENU;
  mdicreate : TMDICreateStruct;
  hwndFrame : HWND;

function WndProc(hWnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
var
  ClientStruct: TClientCreateStruct;
begin
  Result := 0;
  case uMsg of
    WM_CREATE:
      begin
        hMainMenu := GetMenu(hWnd);
        ClientStruct.hWindowMenu  := GetSubMenu(hMainMenu, 1);
        ClientStruct.idFirstChild := 100;
        hwndClient := CreateWindowEx(0, MDIClientName, nil,
          WS_CHILD or WS_VISIBLE or WS_CLIPCHILDREN, Integer(CW_USEDEFAULT),
          Integer(CW_USEDEFAULT), Integer(CW_USEDEFAULT),
          Integer(CW_USEDEFAULT), hWnd, 0, HInstance, @ClientStruct);
        {--------------------------------------}
        { Initialize the TMDICreateStruct      }
        {--------------------------------------}
        mdicreate.szClass := MDIChildClassName;
        mdicreate.szTitle := MDIChildTitle;
        mdicreate.hOwner  := HInstance;
        mdicreate.x       := Integer(CW_USEDEFAULT);
        mdicreate.y       := Integer(CW_USEDEFAULT);
        mdicreate.cx      := Integer(CW_USEDEFAULT);
        mdicreate.cy      := Integer(CW_USEDEFAULT);
      end;
    WM_COMMAND:
      if lParam = 0 then
      begin
        case LoWord(wParam) of
          IDM_EXIT:
            SendMessage(hWnd, WM_CLOSE, 0, 0);
          IDM_TILEHORZ:
            SendMessage(hwndClient, WM_MDITILE, MDITILE_HORIZONTAL, 0);
          IDM_TILEVERT:
            SendMessage(hwndClient, WM_MDITILE, MDITILE_VERTICAL, 0);
          IDM_CASCADE:
            SendMessage(hwndClient, WM_MDICASCADE, MDITILE_SKIPDISABLED, 0);
          IDM_NEW:
            SendMessage(hwndClient, WM_MDICREATE, 0, Integer(@mdicreate));
          IDM_CLOSE:
            SendMessage(SendMessage(hwndClient, WM_MDIGETACTIVE, 0, 0), WM_CLOSE, 0, 0);
        else
          Result := DefFrameProc(hWnd, hwndClient, uMsg, wParam, lParam);
        end;
      end;
    WM_DESTROY:
      PostQuitMessage(0);
  else
    Result := DefFrameProc(hWnd, hwndClient, uMsg, wParam, lParam);
  end;
end;

function ChildProc(hChild: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  Result := 0;
  case uMsg of
    WM_MDIACTIVATE:
      begin
        if lParam = Integer(hChild) then
        begin
          SendMessage(hwndClient, WM_MDISETMENU, hChildMenu,
            GetSubMenu(hChildMenu, 1));
        end else begin
	  SendMessage(hwndClient, WM_MDISETMENU, hMainMenu,
            GetSubMenu(hMainMenu, 1));
        end;
        DrawMenuBar(hwndFrame);
      end;
    WM_CLOSE:
      if MessageBox(hChild, ClosePromptMessage, AppName, MB_YESNO) = IDYES then
        SendMessage(hwndClient, WM_MDIDESTROY, hChild, 0);
  else
    Result := DefMDIChildProc(hChild, uMsg, wParam, lParam);
  end;
end;


var
  wc: TWndClassEx = (
    cbSize       : SizeOf(TWndClassEx);
    style        : CS_HREDRAW or CS_VREDRAW;
    lpfnWndProc  : @WndProc;
    cbClsExtra   : 0;
    cbWndExtra   : 0;
    hbrBackground: COLOR_APPWORKSPACE;
    lpszMenuName : PChar(IDR_MAINMENU);
    lpszClassName: ClassName;
    hIconSm      : 0;
  );
  msg: TMsg;

begin
  {--------------------------------------------}
  { Register the frame window class            }
  {--------------------------------------------}
  wc.hInstance := HInstance;
  wc.hIcon     := LoadIcon(0, IDI_APPLICATION);
  wc.hCursor   := LoadCursor(0, IDC_ARROW);
  RegisterClassEx(wc);
  {-----------------------------------------------}
  { Register the MDI child window class           }
  {-----------------------------------------------}
  wc.lpfnWndProc   := @ChildProc;
  wc.hbrBackground := COLOR_WINDOW + 1;
  wc.lpszClassName := MDIChildClassName;
  RegisterClassEx(wc);
  hwndFrame := CreateWindowEx(0, ClassName, AppName,
    WS_OVERLAPPEDWINDOW or WS_CLIPCHILDREN, Integer(CW_USEDEFAULT),
    Integer(CW_USEDEFAULT), Integer(CW_USEDEFAULT), Integer(CW_USEDEFAULT),
    0, 0, HInstance, nil);
  hChildMenu := LoadMenu(HInstance, PChar(IDR_CHILDMENU));
  ShowWindow(hwndFrame, SW_SHOWNORMAL);
  UpdateWindow(hwndFrame);
  while True do
  begin
    if not GetMessage(msg, 0, 0, 0) then
      Break;
    if not TranslateMDISysAccel(hwndClient, msg) then
    begin
      TranslateMessage(msg);
      DispatchMessage(msg);
    end;
  end;
  DestroyMenu(hChildMenu);
  ExitCode := msg.wParam;
end.
